@@ -1,5 +1,5 @@
 from abc import ABC, abstractmethod
-from typing import Type
+from typing import Type, Optional, Any, Dict
 
 from bauh.api.abstract.cache import MemoryCache
 from bauh.api.abstract.model import SoftwarePackage
@@ -29,6 +29,12 @@ class DiskCacheLoader:
         """
         pass
 
+    def read(self, pkg: SoftwarePackage) -> Optional[Dict[str, Any]]:
+        """
+        returns the cached data from the given package
+        """
+        pass
+
 
 class DiskCacheLoaderFactory(ABC):
 
