@@ -2,7 +2,7 @@ from packaging.version import Version
 
 from bauh.api.abstract.model import SoftwarePackage, PackageStatus
 from bauh.commons import resource
-from bauh.gems.flatpak import ROOT_DIR, VERSION_1_2, VERSION_1_5
+from bauh.gems.flatpak import ROOT_DIR, VERSION_1_2
 from bauh.view.util.translation import I18n
 
 
@@ -125,7 +125,16 @@ class FlatpakApplication(SoftwarePackage):
 
     def __eq__(self, other):
         if isinstance(other, FlatpakApplication):
-            return self.id == other.id and self.installation == other.installation and self.branch == other.branch
+            return self.id == other.id and self.installation == other.installation and self.branch == other.branch \
+                   and self.runtime == other.runtime and self.partial == other.partial and \
+                   self.update_component == other.update_component
+
+    def __hash__(self) -> int:
+        hash_sum = 0
+        for attr in ('id', 'installation', 'branch', 'runtime', 'partial', 'update_component'):
+            hash_sum += hash(getattr(self, attr))
+
+        return hash_sum
 
     def get_disk_icon_path(self) -> str:
         if not self.runtime:
@@ -137,9 +146,22 @@ class FlatpakApplication(SoftwarePackage):
         else:
             return f'{self.installation}/{self.ref}'
 
+    def can_be_installed(self) -> bool:
+        return not self.update_component and not self.installed
+
+    def can_be_updated(self) -> bool:
+        return self.update_component or super(FlatpakApplication, self).can_be_updated()
+
     def can_be_uninstalled(self) -> bool:
         return not self.update_component and super(FlatpakApplication, self).can_be_uninstalled()
 
     def update_ref(self):
         if self.id and self.arch and self.branch:
             self.ref = f'{self.id}/{self.arch}/{self.branch}'
+
+    def __repr__(self) -> str:
+        return f'Flatpak (id={self.id}, branch={self.branch}, origin={self.origin}, installation={self.installation},' \
+               f' partial={self.partial}, update_component={self.update_component})'
+
+    def __str__(self):
+        return self.__repr__()
